@@ -1,20 +1,25 @@
-use std::borrow::Cow;
-use std::io::Write;
-use std::sync::mpsc::Sender;
-use std::sync::{Arc, Mutex};
-use std::{cmp, fmt, fs, io};
+use std::{
+    borrow::Cow,
+    cmp, fmt, fs, io,
+    io::Write,
+    sync::{mpsc::Sender, Arc, Mutex},
+};
+
+#[cfg(feature = "date-based")]
+use std::path::{Path, PathBuf};
 
 #[cfg(all(not(windows), feature = "syslog-4"))]
 use std::collections::HashMap;
 
-use log::{self, Log};
+use log::Log;
 
-use {log_impl, Filter, FormatCallback, Formatter};
+use crate::{log_impl, Filter, FormatCallback, Formatter};
+
+#[cfg(feature = "date-based")]
+use crate::log_impl::DateBasedState;
 
-#[cfg(all(not(windows), feature = "syslog-3"))]
-use syslog_3;
 #[cfg(all(not(windows), feature = "syslog-4"))]
-use {Syslog4Rfc3164Logger, Syslog4Rfc5424Logger};
+use crate::{Syslog4Rfc3164Logger, Syslog4Rfc5424Logger};
 
 /// The base dispatch logger.
 ///
@@ -31,10 +36,6 @@ use {Syslog4Rfc3164Logger, Syslog4Rfc5424Logger};
 ///
 /// ```no_run
 /// # // no_run because this creates log files.
-/// #[macro_use]
-/// extern crate log;
-/// extern crate fern;
-///
 /// use std::{fs, io};
 ///
 /// # fn setup_logger() -> Result<(), fern::InitError> {
@@ -65,12 +66,14 @@ use {Syslog4Rfc3164Logger, Syslog4Rfc5424Logger};
 ///             // `log_file(x)` equates to
 ///             // `OpenOptions::new().write(true).append(true).create(true).open(x)`
 ///             .chain(fern::log_file("persistent-log.log")?)
-///             .chain(fs::OpenOptions::new()
-///                 .write(true)
-///                 .create(true)
-///                 .truncate(true)
-///                 .create(true)
-///                 .open("/tmp/temp.log")?),
+///             .chain(
+///                 fs::OpenOptions::new()
+///                     .write(true)
+///                     .create(true)
+///                     .truncate(true)
+///                     .create(true)
+///                     .open("/tmp/temp.log")?,
+///             ),
 ///     )
 ///     .chain(
 ///         fern::Dispatch::new()
@@ -185,8 +188,7 @@ impl Dispatch {
     /// Example usage:
     ///
     /// ```
-    /// fern::Dispatch::new()
-    ///     .chain(fern::Dispatch::new().chain(std::io::stdout()))
+    /// fern::Dispatch::new().chain(fern::Dispatch::new().chain(std::io::stdout()))
     ///     # .into_log();
     /// ```
     #[inline]
@@ -210,12 +212,8 @@ impl Dispatch {
     /// Example usage:
     ///
     /// ```
-    /// # extern crate log;
-    /// # extern crate fern;
-    /// #
     /// # fn main() {
-    /// fern::Dispatch::new()
-    ///     .level(log::LevelFilter::Info)
+    /// fern::Dispatch::new().level(log::LevelFilter::Info)
     ///     # .into_log();
     /// # }
     /// ```
@@ -245,9 +243,6 @@ impl Dispatch {
     /// excluded:
     ///
     /// ```
-    /// # extern crate log;
-    /// # extern crate fern;
-    /// #
     /// # fn main() {
     /// fern::Dispatch::new()
     ///     .level(log::LevelFilter::Trace)
@@ -262,16 +257,12 @@ impl Dispatch {
     /// rest of the program at info level:
     ///
     /// ```
-    /// # extern crate log;
-    /// # extern crate fern;
-    /// #
     /// fn setup_logging<T, I>(verbose_modules: T) -> Result<(), fern::InitError>
     /// where
     ///     I: AsRef<str>,
     ///     T: IntoIterator<Item = I>,
     /// {
-    ///     let mut config = fern::Dispatch::new()
-    ///         .level(log::LevelFilter::Info);
+    ///     let mut config = fern::Dispatch::new().level(log::LevelFilter::Info);
     ///
     ///     for module_name in verbose_modules {
     ///         config = config.level_for(
@@ -326,9 +317,6 @@ impl Dispatch {
     /// This sends error level messages to stderr and others to stdout.
     ///
     /// ```
-    /// # extern crate log;
-    /// # extern crate fern;
-    /// #
     /// # fn main() {
     /// fern::Dispatch::new()
     ///     .level(log::LevelFilter::Info)
@@ -369,9 +357,6 @@ impl Dispatch {
     /// program only opens "file.log" once.
     ///
     /// ```no_run
-    /// # extern crate log;
-    /// # extern crate fern;
-    /// #
     /// # fn setup_logger() -> Result<(), fern::InitError> {
     ///
     /// let file_out = fern::Dispatch::new()
@@ -408,7 +393,7 @@ impl Dispatch {
 
         SharedDispatch {
             inner: Arc::new(dispatch),
-            min_level: min_level,
+            min_level,
         }
     }
 
@@ -433,36 +418,44 @@ impl Dispatch {
                 OutputInner::Stdout { stream, line_sep } => {
                     max_child_level = log::LevelFilter::Trace;
                     Some(log_impl::Output::Stdout(log_impl::Stdout {
-                        stream: stream,
-                        line_sep: line_sep,
+                        stream,
+                        line_sep,
                     }))
                 }
                 OutputInner::Stderr { stream, line_sep } => {
                     max_child_level = log::LevelFilter::Trace;
                     Some(log_impl::Output::Stderr(log_impl::Stderr {
-                        stream: stream,
-                        line_sep: line_sep,
+                        stream,
+                        line_sep,
                     }))
                 }
                 OutputInner::File { stream, line_sep } => {
                     max_child_level = log::LevelFilter::Trace;
                     Some(log_impl::Output::File(log_impl::File {
                         stream: Mutex::new(io::BufWriter::new(stream)),
-                        line_sep: line_sep,
+                        line_sep,
                     }))
                 }
                 OutputInner::Writer { stream, line_sep } => {
                     max_child_level = log::LevelFilter::Trace;
                     Some(log_impl::Output::Writer(log_impl::Writer {
                         stream: Mutex::new(stream),
-                        line_sep: line_sep,
+                        line_sep,
+                    }))
+                }
+                #[cfg(all(not(windows), feature = "reopen-03"))]
+                OutputInner::Reopen { stream, line_sep } => {
+                    max_child_level = log::LevelFilter::Trace;
+                    Some(log_impl::Output::Reopen(log_impl::Reopen {
+                        stream: Mutex::new(stream),
+                        line_sep,
                     }))
                 }
                 OutputInner::Sender { stream, line_sep } => {
                     max_child_level = log::LevelFilter::Trace;
                     Some(log_impl::Output::Sender(log_impl::Sender {
                         stream: Mutex::new(stream),
-                        line_sep: line_sep,
+                        line_sep,
                     }))
                 }
                 #[cfg(all(not(windows), feature = "syslog-3"))]
@@ -519,6 +512,31 @@ impl Dispatch {
                     max_child_level = log::LevelFilter::Trace;
                     Some(log_impl::Output::OtherStatic(child_log))
                 }
+                #[cfg(feature = "date-based")]
+                OutputInner::DateBased { config } => {
+                    max_child_level = log::LevelFilter::Trace;
+
+                    let config = log_impl::DateBasedConfig::new(
+                        config.line_sep,
+                        config.file_prefix,
+                        config.file_suffix,
+                        if config.utc_time {
+                            log_impl::ConfiguredTimezone::Utc
+                        } else {
+                            log_impl::ConfiguredTimezone::Local
+                        },
+                    );
+
+                    let computed_suffix = config.compute_current_suffix();
+
+                    // ignore errors - we'll just retry later.
+                    let initial_file = config.open_current_log_file(&computed_suffix).ok();
+
+                    Some(log_impl::Output::DateBased(log_impl::DateBased {
+                        config,
+                        state: Mutex::new(DateBasedState::new(computed_suffix, initial_file)),
+                    }))
+                }
             })
             .collect();
 
@@ -557,9 +575,6 @@ impl Dispatch {
     /// Example usage:
     ///
     /// ```
-    /// # extern crate log;
-    /// # extern crate fern;
-    /// #
     /// # fn main() {
     /// let (min_level, log) = fern::Dispatch::new()
     ///     .level(log::LevelFilter::Info)
@@ -569,7 +584,7 @@ impl Dispatch {
     /// assert_eq!(min_level, log::LevelFilter::Info);
     /// # }
     /// ```
-    pub fn into_log(self) -> (log::LevelFilter, Box<log::Log>) {
+    pub fn into_log(self) -> (log::LevelFilter, Box<dyn log::Log>) {
         let (level, logger) = self.into_dispatch();
         if level == log::LevelFilter::Off {
             (level, Box::new(log_impl::Null))
@@ -615,7 +630,14 @@ enum OutputInner {
     },
     /// Writes all messages to the writer with `line_sep` separator.
     Writer {
-        stream: Box<Write + Send>,
+        stream: Box<dyn Write + Send>,
+        line_sep: Cow<'static, str>,
+    },
+    /// Writes all messages to the reopen::Reopen file with `line_sep`
+    /// separator.
+    #[cfg(all(not(windows), feature = "reopen-03"))]
+    Reopen {
+        stream: reopen::Reopen<fs::File>,
         line_sep: Cow<'static, str>,
     },
     /// Writes all messages to mpst::Sender with `line_sep` separator.
@@ -628,12 +650,12 @@ enum OutputInner {
     /// Passes all messages to other dispatch that's shared.
     SharedDispatch(SharedDispatch),
     /// Passes all messages to other logger.
-    OtherBoxed(Box<Log>),
+    OtherBoxed(Box<dyn Log>),
     /// Passes all messages to other logger.
-    OtherStatic(&'static Log),
+    OtherStatic(&'static dyn Log),
     /// Passes all messages to the syslog.
     #[cfg(all(not(windows), feature = "syslog-3"))]
-    Syslog3(syslog_3::Logger),
+    Syslog3(syslog3::Logger),
     /// Passes all messages to the syslog.
     #[cfg(all(not(windows), feature = "syslog-4"))]
     Syslog4Rfc3164(Syslog4Rfc3164Logger),
@@ -642,13 +664,16 @@ enum OutputInner {
     Syslog4Rfc5424 {
         logger: Syslog4Rfc5424Logger,
         transform: Box<
-            Fn(&log::Record) -> (i32, HashMap<String, HashMap<String, String>>, String)
+            dyn Fn(&log::Record) -> (i32, HashMap<String, HashMap<String, String>>, String)
                 + Sync
                 + Send,
         >,
     },
     /// Panics with messages text for all messages.
     Panic,
+    /// File logger with custom date and timestamp suffix in file name.
+    #[cfg(feature = "date-based")]
+    DateBased { config: DateBased },
 }
 
 /// Logger which will panic whenever anything is logged. The panic
@@ -663,15 +688,13 @@ enum OutputInner {
 /// message is sent.
 ///
 /// ```
-/// # extern crate fern;
-/// # extern crate log;
 /// fern::Dispatch::new()
 ///     // format, etc.
 ///     .chain(std::io::stdout())
 ///     .chain(
 ///         fern::Dispatch::new()
 ///             .level(log::LevelFilter::Error)
-///             .chain(fern::Panic)
+///             .chain(fern::Panic),
 ///     )
 ///     # /*
 ///     .apply()?;
@@ -685,8 +708,6 @@ enum OutputInner {
 /// messages.
 ///
 /// ```no_run
-/// # extern crate fern;
-/// # extern crate log;
 /// fn setup_panic_logging() {
 ///     fern::Dispatch::new()
 ///         .level(log::LevelFilter::Warn)
@@ -715,16 +736,16 @@ impl From<SharedDispatch> for Output {
     }
 }
 
-impl From<Box<Log>> for Output {
+impl From<Box<dyn Log>> for Output {
     /// Creates an output logger forwarding all messages to the custom logger.
-    fn from(log: Box<Log>) -> Self {
+    fn from(log: Box<dyn Log>) -> Self {
         Output(OutputInner::OtherBoxed(log))
     }
 }
 
-impl From<&'static Log> for Output {
+impl From<&'static dyn Log> for Output {
     /// Creates an output logger forwarding all messages to the custom logger.
-    fn from(log: &'static Log) -> Self {
+    fn from(log: &'static dyn Log) -> Self {
         Output(OutputInner::OtherStatic(log))
     }
 }
@@ -742,14 +763,14 @@ impl From<fs::File> for Output {
     }
 }
 
-impl From<Box<Write + Send>> for Output {
+impl From<Box<dyn Write + Send>> for Output {
     /// Creates an output logger which writes all messages to the writer with
     /// `\n` as the separator.
     ///
     /// This does no buffering and it is up to the writer to do buffering as
     /// needed (eg. wrap it in `BufWriter`). However, flush is called after
     /// each log record.
-    fn from(writer: Box<Write + Send>) -> Self {
+    fn from(writer: Box<dyn Write + Send>) -> Self {
         Output(OutputInner::Writer {
             stream: writer,
             line_sep: "\n".into(),
@@ -757,12 +778,24 @@ impl From<Box<Write + Send>> for Output {
     }
 }
 
+#[cfg(all(not(windows), feature = "reopen-03"))]
+impl From<reopen::Reopen<fs::File>> for Output {
+    /// Creates an output logger which writes all messages to the file contained
+    /// in the Reopen struct, using `\n` as the separator.
+    fn from(reopen: reopen::Reopen<fs::File>) -> Self {
+        Output(OutputInner::Reopen {
+            stream: reopen,
+            line_sep: "\n".into(),
+        })
+    }
+}
+
 impl From<io::Stdout> for Output {
     /// Creates an output logger which writes all messages to stdout with the
     /// given handle and `\n` as the separator.
     fn from(stream: io::Stdout) -> Self {
         Output(OutputInner::Stdout {
-            stream: stream,
+            stream,
             line_sep: "\n".into(),
         })
     }
@@ -773,7 +806,7 @@ impl From<io::Stderr> for Output {
     /// given handle and `\n` as the separator.
     fn from(stream: io::Stderr) -> Self {
         Output(OutputInner::Stderr {
-            stream: stream,
+            stream,
             line_sep: "\n".into(),
         })
     }
@@ -786,14 +819,14 @@ impl From<Sender<String>> for Output {
     /// All messages sent to the mpsc channel are suffixed with '\n'.
     fn from(stream: Sender<String>) -> Self {
         Output(OutputInner::Sender {
-            stream: stream,
+            stream,
             line_sep: "\n".into(),
         })
     }
 }
 
 #[cfg(all(not(windows), feature = "syslog-3"))]
-impl From<syslog_3::Logger> for Output {
+impl From<syslog3::Logger> for Output {
     /// Creates an output logger which writes all messages to the given syslog
     /// output.
     ///
@@ -801,13 +834,13 @@ impl From<syslog_3::Logger> for Output {
     /// informational, warn => warning, and error => error.
     ///
     /// This requires the `"syslog-3"` feature.
-    fn from(log: syslog_3::Logger) -> Self {
+    fn from(log: syslog3::Logger) -> Self {
         Output(OutputInner::Syslog3(log))
     }
 }
 
 #[cfg(all(not(windows), feature = "syslog-3"))]
-impl From<Box<syslog_3::Logger>> for Output {
+impl From<Box<syslog3::Logger>> for Output {
     /// Creates an output logger which writes all messages to the given syslog
     /// output.
     ///
@@ -820,7 +853,7 @@ impl From<Box<syslog_3::Logger>> for Output {
     /// identical to that created by passing a raw `syslog::Logger`.
     ///
     /// This requires the `"syslog-3"` feature.
-    fn from(log: Box<syslog_3::Logger>) -> Self {
+    fn from(log: Box<syslog3::Logger>) -> Self {
         Output(OutputInner::Syslog3(*log))
     }
 }
@@ -860,8 +893,7 @@ impl Output {
     ///
     /// ```no_run
     /// # fn setup_logger() -> Result<(), fern::InitError> {
-    /// fern::Dispatch::new()
-    ///     .chain(std::fs::File::create("log")?)
+    /// fern::Dispatch::new().chain(std::fs::File::create("log")?)
     ///     # .into_log();
     /// # Ok(())
     /// # }
@@ -871,8 +903,7 @@ impl Output {
     ///
     /// ```no_run
     /// # fn setup_logger() -> Result<(), fern::InitError> {
-    /// fern::Dispatch::new()
-    ///     .chain(fern::log_file("log")?)
+    /// fern::Dispatch::new().chain(fern::log_file("log")?)
     ///     # .into_log();
     /// # Ok(())
     /// # }
@@ -884,8 +915,7 @@ impl Output {
     ///
     /// ```no_run
     /// # fn setup_logger() -> Result<(), fern::InitError> {
-    /// fern::Dispatch::new()
-    ///     .chain(fern::Output::file(fern::log_file("log")?, "\r\n"))
+    /// fern::Dispatch::new().chain(fern::Output::file(fern::log_file("log")?, "\r\n"))
     ///     # .into_log();
     /// # Ok(())
     /// # }
@@ -929,8 +959,7 @@ impl Output {
     /// # fn setup_logger() -> Result<(), fern::InitError> {
     /// let writer = Box::new(std::io::Cursor::new(Vec::<u8>::new()));
     ///
-    /// fern::Dispatch::new()
-    ///     .chain(fern::Output::writer(writer, "\r\n"))
+    /// fern::Dispatch::new().chain(fern::Output::writer(writer, "\r\n"))
     ///     # .into_log();
     /// #     Ok(())
     /// # }
@@ -939,21 +968,56 @@ impl Output {
     /// ```
     ///
     /// [`Dispatch::chain`]: struct.Dispatch.html#method.chain
-    pub fn writer<T: Into<Cow<'static, str>>>(writer: Box<Write + Send>, line_sep: T) -> Self {
+    pub fn writer<T: Into<Cow<'static, str>>>(writer: Box<dyn Write + Send>, line_sep: T) -> Self {
         Output(OutputInner::Writer {
             stream: writer,
             line_sep: line_sep.into(),
         })
     }
 
+    /// Returns a reopenable logger, i.e., handling SIGHUP.
+    ///
+    /// If the default separator of `\n` is acceptable, a `Reopen`
+    /// instance can be passed into [`Dispatch::chain`] directly.
+    ///
+    /// ```no_run
+    /// use std::fs::OpenOptions;
+    /// # fn setup_logger() -> Result<(), fern::InitError> {
+    /// let reopenable = reopen::Reopen::new(Box::new(|| {
+    ///     OpenOptions::new()
+    ///         .create(true)
+    ///         .write(true)
+    ///         .append(true)
+    ///         .open("/tmp/output.log")
+    /// }))
+    /// .unwrap();
+    ///
+    /// fern::Dispatch::new().chain(fern::Output::reopen(reopenable, "\n"))
+    ///     # .into_log();
+    /// #     Ok(())
+    /// # }
+    /// #
+    /// # fn main() { setup_logger().expect("failed to set up logger"); }
+    /// ```
+    /// [`Dispatch::chain`]: struct.Dispatch.html#method.chain
+    #[cfg(all(not(windows), feature = "reopen-03"))]
+    pub fn reopen<T: Into<Cow<'static, str>>>(
+        reopen: reopen::Reopen<fs::File>,
+        line_sep: T,
+    ) -> Self {
+        Output(OutputInner::Reopen {
+            stream: reopen,
+            line_sep: line_sep.into(),
+        })
+    }
+
     /// Returns an stdout logger using a custom separator.
     ///
     /// If the default separator of `\n` is acceptable, an `io::Stdout`
     /// instance can be passed into `Dispatch::chain()` directly.
     ///
     /// ```
-    /// fern::Dispatch::new()
-    ///     .chain(std::io::stdout())
+    /// fern::Dispatch::new().chain(std::io::stdout())
     ///     # .into_log();
     /// ```
     ///
@@ -979,16 +1043,14 @@ impl Output {
     /// instance can be passed into `Dispatch::chain()` directly.
     ///
     /// ```
-    /// fern::Dispatch::new()
-    ///     .chain(std::io::stderr())
+    /// fern::Dispatch::new().chain(std::io::stderr())
     ///     # .into_log();
     /// ```
     ///
     /// Example usage:
     ///
     /// ```
-    /// fern::Dispatch::new()
-    ///     .chain(fern::Output::stderr("\n\n\n"))
+    /// fern::Dispatch::new().chain(fern::Output::stderr("\n\n\n"))
     ///     # .into_log();
     /// ```
     pub fn stderr<T: Into<Cow<'static, str>>>(line_sep: T) -> Self {
@@ -1011,8 +1073,7 @@ impl Output {
     /// use std::sync::mpsc::channel;
     ///
     /// let (tx, rx) = channel();
-    /// fern::Dispatch::new()
-    ///     .chain(tx)
+    /// fern::Dispatch::new().chain(tx)
     ///     # .into_log();
     /// ```
     pub fn sender<T: Into<Cow<'static, str>>>(sender: Sender<String>, line_sep: T) -> Self {
@@ -1050,18 +1111,18 @@ impl Output {
         })
     }
 
-    /// Returns a logger which simply calls the given function with each message.
+    /// Returns a logger which simply calls the given function with each
+    /// message.
     ///
     /// The function will be called inline in the thread the log occurs on.
     ///
     /// Example usage:
     ///
     /// ```
-    /// fern::Dispatch::new()
-    ///     .chain(fern::Output::call(|record| {
-    ///         // this is mundane, but you can do anything here.
-    ///         println!("{}", record.args());
-    ///     }))
+    /// fern::Dispatch::new().chain(fern::Output::call(|record| {
+    ///     // this is mundane, but you can do anything here.
+    ///     println!("{}", record.args());
+    /// }))
     ///     # .into_log();
     /// ```
     pub fn call<F>(func: F) -> Self
@@ -1082,7 +1143,7 @@ impl Output {
             fn flush(&self) {}
         }
 
-        Self::from(Box::new(CallShim(func)) as Box<log::Log>)
+        Self::from(Box::new(CallShim(func)) as Box<dyn log::Log>)
     }
 }
 
@@ -1160,6 +1221,12 @@ impl fmt::Debug for OutputInner {
                 .field("stream", &"<unknown writer>")
                 .field("line_sep", line_sep)
                 .finish(),
+            #[cfg(all(not(windows), feature = "reopen-03"))]
+            OutputInner::Reopen { ref line_sep, .. } => f
+                .debug_struct("Output::Reopen")
+                .field("stream", &"<unknown reopen file>")
+                .field("line_sep", line_sep)
+                .finish(),
             OutputInner::Sender {
                 ref stream,
                 ref line_sep,
@@ -1199,6 +1266,11 @@ impl fmt::Debug for OutputInner {
                 .field(&"<boxed logger>")
                 .finish(),
             OutputInner::Panic => f.debug_tuple("Output::Panic").finish(),
+            #[cfg(feature = "date-based")]
+            OutputInner::DateBased { ref config } => f
+                .debug_struct("Output::DateBased")
+                .field("config", config)
+                .finish(),
         }
     }
 }
@@ -1208,3 +1280,174 @@ impl fmt::Debug for Output {
         self.0.fmt(f)
     }
 }
+
+/// This is used to generate log file suffixed based on date, hour, and minute.
+///
+/// The log file will be rotated automatically when the date changes.
+#[derive(Debug)]
+#[cfg(feature = "date-based")]
+pub struct DateBased {
+    file_prefix: PathBuf,
+    file_suffix: Cow<'static, str>,
+    line_sep: Cow<'static, str>,
+    utc_time: bool,
+}
+
+#[cfg(feature = "date-based")]
+impl DateBased {
+    /// Create new date-based file logger with the given file prefix and
+    /// strftime-based suffix pattern.
+    ///
+    /// On initialization, fern will create a file with the suffix formatted
+    /// with the current time (either utc or local, see below). Each time a
+    /// record is logged, the format is checked against the current time, and if
+    /// the time has changed, the old file is closed and a new one opened.
+    ///
+    /// `file_suffix` will be interpreted as an `strftime` format. See
+    /// [`chrono::format::strftime`] for more information.
+    ///
+    /// `file_prefix` may be a full file path, and will be prepended to the
+    /// suffix to create the final file.
+    ///
+    /// Note that no separator will be placed in between `file_name` and
+    /// `file_suffix_pattern`. So if you call `DateBased::new("hello",
+    /// "%Y")`, the result will be a filepath `hello2019`.
+    ///
+    /// By default, this will use local time. For UTC time instead, use the
+    /// [`.utc_time()`][DateBased::utc_time] method after creating.
+    ///
+    /// By default, this will use `\n` as a line separator. For a custom
+    /// separator, use the [`.line_sep`][DateBased::line_sep] method
+    /// after creating.
+    ///
+    /// # Examples
+    ///
+    /// Containing the date (year, month and day):
+    ///
+    /// ```
+    /// // logs/2019-10-23-my-program.log
+    /// let log = fern::DateBased::new("logs/", "%Y-%m-%d-my-program.log");
+    ///
+    /// // program.log.23102019
+    /// let log = fern::DateBased::new("my-program.log.", "%d%m%Y");
+    /// ```
+    ///
+    /// Containing the hour:
+    ///
+    /// ```
+    /// // logs/2019-10-23 13 my-program.log
+    /// let log = fern::DateBased::new("logs/", "%Y-%m-%d %H my-program.log");
+    ///
+    /// // program.log.2310201913
+    /// let log = fern::DateBased::new("my-program.log.", "%d%m%Y%H");
+    /// ```
+    ///
+    /// Containing the minute:
+    ///
+    /// ```
+    /// // logs/2019-10-23 13 my-program.log
+    /// let log = fern::DateBased::new("logs/", "%Y-%m-%d %H my-program.log");
+    ///
+    /// // program.log.2310201913
+    /// let log = fern::DateBased::new("my-program.log.", "%d%m%Y%H");
+    /// ```
+    ///
+    /// UNIX time, or seconds since 00:00 Jan 1st 1970:
+    ///
+    /// ```
+    /// // logs/1571822854-my-program.log
+    /// let log = fern::DateBased::new("logs/", "%s-my-program.log");
+    ///
+    /// // program.log.1571822854
+    /// let log = fern::DateBased::new("my-program.log.", "%s");
+    /// ```
+    ///
+    /// Hourly, using UTC time:
+    ///
+    /// ```
+    /// // logs/2019-10-23 23 my-program.log
+    /// let log = fern::DateBased::new("logs/", "%Y-%m-%d %H my-program.log").utc_time();
+    ///
+    /// // program.log.2310201923
+    /// let log = fern::DateBased::new("my-program.log.", "%d%m%Y%H").utc_time();
+    /// ```
+    ///
+    /// [`chrono::format::strftime`]: https://docs.rs/chrono/0.4.6/chrono/format/strftime/index.html
+    pub fn new<T, U>(file_prefix: T, file_suffix: U) -> Self
+    where
+        T: AsRef<Path>,
+        U: Into<Cow<'static, str>>,
+    {
+        DateBased {
+            utc_time: false,
+            file_prefix: file_prefix.as_ref().to_owned(),
+            file_suffix: file_suffix.into(),
+            line_sep: "\n".into(),
+        }
+    }
+
+    /// Changes the line separator this logger will use.
+    ///
+    /// The default line separator is `\n`.
+    ///
+    /// # Examples
+    ///
+    /// Using a windows line separator:
+    ///
+    /// ```
+    /// let log = fern::DateBased::new("logs", "%s.log").line_sep("\r\n");
+    /// ```
+    pub fn line_sep<T>(mut self, line_sep: T) -> Self
+    where
+        T: Into<Cow<'static, str>>,
+    {
+        self.line_sep = line_sep.into();
+        self
+    }
+
+    /// Orients this log file suffix formatting to use UTC time.
+    ///
+    /// The default is local time.
+    ///
+    /// # Examples
+    ///
+    /// This will use UTC time to determine the date:
+    ///
+    /// ```
+    /// // program.log.2310201923
+    /// let log = fern::DateBased::new("my-program.log.", "%d%m%Y%H").utc_time();
+    /// ```
+    pub fn utc_time(mut self) -> Self {
+        self.utc_time = true;
+        self
+    }
+
+    /// Orients this log file suffix formatting to use local time.
+    ///
+    /// This is the default option.
+    ///
+    /// # Examples
+    ///
+    /// This log file will use local time - the latter method call overrides the
+    /// former.
+    ///
+    /// ```
+    /// // program.log.2310201923
+    /// let log = fern::DateBased::new("my-program.log.", "%d%m%Y%H")
+    ///     .utc_time()
+    ///     .local_time();
+    /// ```
+    pub fn local_time(mut self) -> Self {
+        self.utc_time = false;
+        self
+    }
+}
+
+#[cfg(feature = "date-based")]
+impl From<DateBased> for Output {
+    /// Create an output logger which defers to the given date-based logger. Use
+    /// configuration methods on [DateBased] to set line separator and filename.
+    fn from(config: DateBased) -> Self {
+        Output(OutputInner::DateBased { config })
+    }
+}
