@@ -1,19 +1,26 @@
-use std::borrow::Cow;
-use std::io::{self, BufWriter, Write};
-use std::sync::mpsc;
-use std::sync::{Arc, Mutex};
-use std::{fmt, fs};
+use std::{
+    borrow::Cow,
+    collections::HashMap,
+    fmt, fs,
+    io::{self, BufWriter, Write},
+    sync::{mpsc, Arc, Mutex},
+};
 
-use std::collections::HashMap;
+#[cfg(feature = "date-based")]
+use std::{
+    ffi::OsString,
+    fs::OpenOptions,
+    path::{Path, PathBuf},
+};
 
 use log::{self, Log};
 
-use {Filter, Formatter};
+use crate::{Filter, Formatter};
 
-#[cfg(all(not(windows), feature = "syslog-3"))]
-use syslog_3;
 #[cfg(all(not(windows), feature = "syslog-4"))]
-use {syslog_4, Syslog4Rfc3164Logger, Syslog4Rfc5424Logger};
+use crate::{Syslog4Rfc3164Logger, Syslog4Rfc5424Logger};
+#[cfg(all(not(windows), feature = "reopen-03"))]
+use reopen;
 
 pub enum LevelConfiguration {
     JustDefault,
@@ -63,10 +70,14 @@ pub enum Output {
     Syslog4Rfc5424(Syslog4Rfc5424),
     Dispatch(Dispatch),
     SharedDispatch(Arc<Dispatch>),
-    OtherBoxed(Box<Log>),
-    OtherStatic(&'static Log),
+    OtherBoxed(Box<dyn Log>),
+    OtherStatic(&'static dyn Log),
     Panic(Panic),
     Writer(Writer),
+    #[cfg(feature = "date-based")]
+    DateBased(DateBased),
+    #[cfg(all(not(windows), feature = "reopen-03"))]
+    Reopen(Reopen),
 }
 
 pub struct Stdout {
@@ -90,13 +101,19 @@ pub struct Sender {
 }
 
 pub struct Writer {
-    pub stream: Mutex<Box<Write + Send>>,
+    pub stream: Mutex<Box<dyn Write + Send>>,
+    pub line_sep: Cow<'static, str>,
+}
+
+#[cfg(all(not(windows), feature = "reopen-03"))]
+pub struct Reopen {
+    pub stream: Mutex<reopen::Reopen<fs::File>>,
     pub line_sep: Cow<'static, str>,
 }
 
 #[cfg(all(not(windows), feature = "syslog-3"))]
 pub struct Syslog3 {
-    pub inner: syslog_3::Logger,
+    pub inner: syslog3::Logger,
 }
 
 #[cfg(all(not(windows), feature = "syslog-4"))]
@@ -108,7 +125,9 @@ pub struct Syslog4Rfc3164 {
 pub struct Syslog4Rfc5424 {
     pub inner: Mutex<Syslog4Rfc5424Logger>,
     pub transform: Box<
-        Fn(&log::Record) -> (i32, HashMap<String, HashMap<String, String>>, String) + Sync + Send,
+        dyn Fn(&log::Record) -> (i32, HashMap<String, HashMap<String, String>>, String)
+            + Sync
+            + Send,
     >,
 }
 
@@ -116,6 +135,100 @@ pub struct Panic;
 
 pub struct Null;
 
+/// File logger with a dynamic time-based name.
+#[derive(Debug)]
+#[cfg(feature = "date-based")]
+pub struct DateBased {
+    pub config: DateBasedConfig,
+    pub state: Mutex<DateBasedState>,
+}
+
+#[derive(Debug)]
+#[cfg(feature = "date-based")]
+pub enum ConfiguredTimezone {
+    Local,
+    Utc,
+}
+
+#[derive(Debug)]
+#[cfg(feature = "date-based")]
+pub struct DateBasedConfig {
+    pub line_sep: Cow<'static, str>,
+    /// This is a Path not an str so it can hold invalid UTF8 paths correctly.
+    pub file_prefix: PathBuf,
+    pub file_suffix: Cow<'static, str>,
+    pub timezone: ConfiguredTimezone,
+}
+
+#[derive(Debug)]
+#[cfg(feature = "date-based")]
+pub struct DateBasedState {
+    pub current_suffix: String,
+    pub file_stream: Option<BufWriter<fs::File>>,
+}
+
+#[cfg(feature = "date-based")]
+impl DateBasedState {
+    pub fn new(current_suffix: String, file_stream: Option<fs::File>) -> Self {
+        DateBasedState {
+            current_suffix,
+            file_stream: file_stream.map(BufWriter::new),
+        }
+    }
+
+    pub fn replace_file(&mut self, new_suffix: String, new_file: Option<fs::File>) {
+        if let Some(mut old) = self.file_stream.take() {
+            let _ = old.flush();
+        }
+        self.current_suffix = new_suffix;
+        self.file_stream = new_file.map(BufWriter::new)
+    }
+}
+
+#[cfg(feature = "date-based")]
+impl DateBasedConfig {
+    pub fn new(
+        line_sep: Cow<'static, str>,
+        file_prefix: PathBuf,
+        file_suffix: Cow<'static, str>,
+        timezone: ConfiguredTimezone,
+    ) -> Self {
+        DateBasedConfig {
+            line_sep,
+            file_prefix,
+            file_suffix,
+            timezone,
+        }
+    }
+
+    pub fn compute_current_suffix(&self) -> String {
+        match self.timezone {
+            ConfiguredTimezone::Utc => chrono::Utc::now().format(&self.file_suffix).to_string(),
+            ConfiguredTimezone::Local => chrono::Local::now().format(&self.file_suffix).to_string(),
+        }
+    }
+
+    pub fn compute_file_path(&self, suffix: &str) -> PathBuf {
+        let mut path = OsString::from(&*self.file_prefix);
+        // use the OsString::push method, not PathBuf::push which would add a path
+        // separator
+        path.push(suffix);
+        path.into()
+    }
+
+    pub fn open_log_file(path: &Path) -> io::Result<fs::File> {
+        OpenOptions::new()
+            .write(true)
+            .create(true)
+            .append(true)
+            .open(path)
+    }
+
+    pub fn open_current_log_file(&self, suffix: &str) -> io::Result<fs::File> {
+        Self::open_log_file(&self.compute_file_path(&suffix))
+    }
+}
+
 impl From<Vec<(Cow<'static, str>, log::LevelFilter)>> for LevelConfiguration {
     fn from(mut levels: Vec<(Cow<'static, str>, log::LevelFilter)>) -> Self {
         // Benchmarked separately: https://gist.github.com/daboross/976978d8200caf86e02acb6805961195
@@ -199,6 +312,10 @@ impl Log for Output {
             Output::Syslog4Rfc5424(ref s) => s.enabled(metadata),
             Output::Panic(ref s) => s.enabled(metadata),
             Output::Writer(ref s) => s.enabled(metadata),
+            #[cfg(feature = "date-based")]
+            Output::DateBased(ref s) => s.enabled(metadata),
+            #[cfg(all(not(windows), feature = "reopen-03"))]
+            Output::Reopen(ref s) => s.enabled(metadata),
         }
     }
 
@@ -220,6 +337,10 @@ impl Log for Output {
             Output::Syslog4Rfc5424(ref s) => s.log(record),
             Output::Panic(ref s) => s.log(record),
             Output::Writer(ref s) => s.log(record),
+            #[cfg(feature = "date-based")]
+            Output::DateBased(ref s) => s.log(record),
+            #[cfg(all(not(windows), feature = "reopen-03"))]
+            Output::Reopen(ref s) => s.log(record),
         }
     }
 
@@ -241,6 +362,10 @@ impl Log for Output {
             Output::Syslog4Rfc5424(ref s) => s.flush(),
             Output::Panic(ref s) => s.flush(),
             Output::Writer(ref s) => s.flush(),
+            #[cfg(feature = "date-based")]
+            Output::DateBased(ref s) => s.flush(),
+            #[cfg(all(not(windows), feature = "reopen-03"))]
+            Output::Reopen(ref s) => s.flush(),
         }
     }
 }
@@ -313,7 +438,8 @@ impl Dispatch {
             && self.filters.iter().all(|f| f(metadata))
     }
 
-    /// Check whether a log with the given metadata would eventually end up outputting something.
+    /// Check whether a log with the given metadata would eventually end up
+    /// outputting something.
     ///
     /// This is recursive, and checks children.
     fn deep_enabled(&self, metadata: &log::Metadata) -> bool {
@@ -444,6 +570,9 @@ macro_rules! writer_log_impl {
 writer_log_impl!(File);
 writer_log_impl!(Writer);
 
+#[cfg(all(not(windows), feature = "reopen-03"))]
+writer_log_impl!(Reopen);
+
 impl Log for Sender {
     fn enabled(&self, _: &log::Metadata) -> bool {
         true
@@ -541,6 +670,61 @@ impl Log for Panic {
     fn flush(&self) {}
 }
 
+#[cfg(feature = "date-based")]
+impl Log for DateBased {
+    fn enabled(&self, _: &log::Metadata) -> bool {
+        true
+    }
+
+    fn log(&self, record: &log::Record) {
+        fallback_on_error(record, |record| {
+            // Formatting first prevents deadlocks on file-logging,
+            // when the process of formatting itself is logged.
+            // note: this is only ever needed if some Debug, Display, or other
+            // formatting trait itself is logging.
+            #[cfg(feature = "meta-logging-in-format")]
+            let msg = format!("{}{}", record.args(), self.config.line_sep);
+
+            let mut state = self.state.lock().unwrap_or_else(|e| e.into_inner());
+
+            // check if log needs to be rotated
+            let new_suffix = self.config.compute_current_suffix();
+            if state.file_stream.is_none() || state.current_suffix != new_suffix {
+                let file_open_result = self.config.open_current_log_file(&new_suffix);
+                match file_open_result {
+                    Ok(file) => {
+                        state.replace_file(new_suffix, Some(file));
+                    }
+                    Err(e) => {
+                        state.replace_file(new_suffix, None);
+                        return Err(e.into());
+                    }
+                }
+            }
+
+            // either just initialized writer above, or already errored out.
+            let writer = state.file_stream.as_mut().unwrap();
+
+            #[cfg(feature = "meta-logging-in-format")]
+            write!(writer, "{}", msg)?;
+            #[cfg(not(feature = "meta-logging-in-format"))]
+            write!(writer, "{}{}", record.args(), self.config.line_sep)?;
+
+            writer.flush()?;
+
+            Ok(())
+        });
+    }
+
+    fn flush(&self) {
+        let mut state = self.state.lock().unwrap_or_else(|e| e.into_inner());
+
+        if let Some(stream) = &mut state.file_stream {
+            let _ = stream.flush();
+        }
+    }
+}
+
 #[inline(always)]
 fn fallback_on_error<F>(record: &log::Record, log_func: F)
 where
@@ -583,7 +767,7 @@ enum LogError {
     Io(io::Error),
     Send(mpsc::SendError<String>),
     #[cfg(all(not(windows), feature = "syslog-4"))]
-    Syslog4(syslog_4::Error),
+    Syslog4(syslog4::Error),
 }
 
 impl fmt::Display for LogError {
@@ -610,8 +794,8 @@ impl From<mpsc::SendError<String>> for LogError {
 }
 
 #[cfg(all(not(windows), feature = "syslog-4"))]
-impl From<syslog_4::Error> for LogError {
-    fn from(error: syslog_4::Error) -> Self {
+impl From<syslog4::Error> for LogError {
+    fn from(error: syslog4::Error) -> Self {
         LogError::Syslog4(error)
     }
 }
@@ -674,9 +858,10 @@ mod test {
                 ("root::sub2::sub2.3::sub2.4", Error),
                 ("root::sub2::sub2.3", Warn),
                 ("root::sub3", Off),
-            ].into_iter()
-                .map(|(k, v)| (k.into(), v))
-                .collect(),
+            ]
+            .into_iter()
+            .map(|(k, v)| (k.into(), v))
+            .collect(),
         );
 
         assert_eq!(config.find_module("root"), Some(Trace));
@@ -728,9 +913,10 @@ mod test {
                 ("root::su", Debug),
                 ("root::su:b2", Info),
                 ("root::sub2", Warn),
-            ].into_iter()
-                .map(|(k, v)| (k.into(), v))
-                .collect(),
+            ]
+            .into_iter()
+            .map(|(k, v)| (k.into(), v))
+            .collect(),
         );
 
         assert_eq!(config.find_module("root"), Some(Trace));
@@ -748,12 +934,8 @@ mod test {
     #[test]
     fn test_level_config_all_chars() {
         let config = LevelConfiguration::Minimal(
-            vec![
-                ("♲", Trace),
-                ("☸", Debug),
-                ("♲::☸", Info),
-                ("♲::\t", Debug),
-            ].into_iter()
+            vec![("♲", Trace), ("☸", Debug), ("♲::☸", Info), ("♲::\t", Debug)]
+                .into_iter()
                 .map(|(k, v)| (k.into(), v))
                 .collect(),
         );
